@@ -1,29 +1,31 @@
-// ClassMapTest.java
-
-/**
- *      Copyright (C) 2008 10gen Inc.
- *  
- *   Licensed under the Apache License, Version 2.0 (the "License");
- *   you may not use this file except in compliance with the License.
- *   You may obtain a copy of the License at
- * 
- *      http://www.apache.org/licenses/LICENSE-2.0
- * 
- *   Unless required by applicable law or agreed to in writing, software
- *   distributed under the License is distributed on an "AS IS" BASIS,
- *   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
- *   See the License for the specific language governing permissions and
- *   limitations under the License.
+/*
+ * Copyright (c) 2008-2014 MongoDB, Inc.
+ *
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
+ *
+ *   http://www.apache.org/licenses/LICENSE-2.0
+ *
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
  */
 
+// ClassMapTest.java
+
 package org.bson.util;
 
-import java.io.Serializable;
-import org.testng.annotations.Test;
+import org.junit.Test;
 
+import java.io.Serializable;
+
+import static org.junit.Assert.assertEquals;
 public class ClassMapTest extends com.mongodb.util.TestCase {
 
-    @org.testng.annotations.Test
+    @Test
     public void test(){
 	// maps Classes to Strings
 	ClassMap<String> m = new ClassMap<String>();
@@ -46,9 +48,4 @@ public class ClassMapTest extends com.mongodb.util.TestCase {
 	m.put(Integer.class, "Integer");
 	assertEquals(m.get(Integer.class), "Integer");
     }
-
-    public static void main( String args[] ){
-	ClassMapTest t = new ClassMapTest();
-	t.runConsole();
-    }
 }
